# Simulando uma variavel com distribuicao binomial #

# Simulando a distribuicao com p variando de 0.1 a 0.9

binom=matrix(nrow=21,ncol=11,NA)

w=seq(0.1,0.9,by=0.1)
for (j in 0:20) {
  for (k in 1:9) {
    binom[j+1,k]=dbinom(j,20,w[k])
  }
}

a=NA
for (i in 1:9){
  a[i]=as.character(round(eval((1-2*(i/10))/sqrt(20*(i/10)*(1-(i/10)))),2)) 
}

par(mfrow=c(3,3))
for (k in 1:9){
  plot(binom[,k],main=paste('Skewness :',a[k]))
  lines(binom[,k])
}

# Como calcular a probilidade de obter s=50 sucessos em N=100 tentativas,
#com probabilidade de sucesso variavel?

# Escrevendo a formula
pbinom1=NA
w=seq(from=0.1,to=0.9,by=0.1)
for (k in 1:9){
    pbinom1[k]=choose(100,50)*(w[k]^50)*((1-w[k])^50)
}
par(mfrow=c(1,2))
plot(pbinom1, main='By formula')
lines(pbinom1, col='red')

# Com comando pronto
pbinom2=NA
for (k in 1:9){
  pbinom2[k]=dbinom(50,100,w[k])
}
plot(pbinom2, main='By command')
lines(pbinom2, col='blue')

# Como calcular a probilidade de obter s sucessos em N=100 tentativas,
#com probabilidade fixa (0.5) e s={0,100}?

# Escrevendo a formula
pbinom=matrix(ncol=1,nrow=101,NA)
for (k in 0:100){
  pbinom[k]=choose(100,k)*(0.5^k)*(0.5^(100-k))
}
par(mfrow=c(1,1))
plot(pbinom)
lines(pbinom,col='red')

# Com comando pronto
for (k in 0:100){
  pbinom[k]=dbinom(k,100,0.5)
}
plot(pbinom)
lines(pbinom,col='blue')

# Simulando convergencia em distribuicao da binomial quando n aumenta

# Distribuicao original

t10=seq(from=1,11,1)
for (k in 0:10){
  t10[k+1]=dbinom(k,10,0.2)
}

t100=seq(from=1,101,1)
for (k in 0:100){
  t100[k+1]=dbinom(k,100,0.2)
}

t1000=seq(from=1,1001,1)
for (k in 1:1000){
  t1000[k+1]=dbinom(k,1000,0.2)
}

t10000=seq(from=1,10001,1)
for (k in 1:10000){
  t10000[k+1]=dbinom(k,10000,0.2)
}

par(mfrow=c(2,2))

plot(t10,main="n=10")
x=t10
curve(dnorm(x,mean=2,sd=sqrt(1.6)),col=2,add=TRUE)

plot(t100,main="n=100", xlim=c(0,40))
x=t100
curve(dnorm(x,mean=20,sd=sqrt(16)),col=2,add=TRUE)

plot(t1000,main="n=10000",
     ylim=c(0,0.05),xlim=c(200-3*sqrt(160),200+3*sqrt(160)))
x=t1000
curve(dnorm(x,mean=200,sd=sqrt(160)),col=2,add=TRUE)

plot(t10000,main="n=10000",
     ylim=c(0,0.015),xlim=c(2000-3*sqrt(1600),2000+3*sqrt(1600)))
x=t10000
curve(dnorm(x,mean=2000,sd=sqrt(1600)),col=2,add=TRUE)

# Lei dos Grandes Numeros
# media e var viram as verdadeiras na populacao assintoticamente

x=matrix(nrow=100000,ncol=5,NA)

set.seed(16042018)
for (j in 1:5){
  x[1:(10^j),j]=rbinom((10^j),10,0.2)
}

xtable=matrix(nrow=2,ncol=5,NA)
for(j in 1:5){
  xtable[1,j]=round(mean(x[1:(10^j),j]),2)
  xtable[2,j]=round(var(x[1:(10^j),j]),2)
  rownames(xtable)=c('Media','Variancia')
  colnames(xtable)=c('n=10','n=100','n=1000','n=10000','n=100000')
}
xtable


# Teorema Central do Limite:

N <- 10000
n <- c(10,100,1000,10000)

# Caso distribuicao das medias de binomiais

s <- matrix(NA,nrow=N,ncol=length(n))

j <- 1

while(j <= N){
  for(i in 1:length(n)){
    x <- rbinom(n[i],10,0.2)
    s[j,i] <- mean(x)
  }
  j <- j+1
}

par(mfrow=c(2,2))

hist(s[,1],freq=FALSE,main="n=10",xlab="x",ylim=c(0,2),breaks=20,xlim=c(min(s[,1]),max(s[,1])))
curve(dnorm(x,mean=2,sd=sqrt(1.6/n[1])),col=2,add=TRUE)
abline(v=mean(s[,1]),col='blue')

hist(s[,2],freq=FALSE,main="n=100",xlab="x",ylim=c(0,6),breaks=20,xlim=c(min(s[,1]),max(s[,1])))
curve(dnorm(x,mean=2,sd=sqrt(1.6/n[2])),col=2,add=TRUE)
abline(v=mean(s[,2]),col='blue')

hist(s[,3],freq=FALSE,main="n=1000",xlab="x",ylim=c(0,20),breaks=20,xlim=c(min(s[,1]),max(s[,1])))
curve(dnorm(x,mean=2,sd=sqrt(1.6/n[3])),col=2,add=TRUE)
abline(v=mean(s[,3]),col='blue')

hist(s[,4],freq=FALSE,main="n=10000",xlab="x",ylim=c(0,60),breaks=20,xlim=c(min(s[,1]),max(s[,1])))
curve(dnorm(x,mean=2,sd=sqrt(1.6/n[4])),col=2,add=TRUE)
abline(v=mean(s[,4]),col='blue')

mtext("Distribuicao Binomial", side = 3, line = -1.5, outer = TRUE, cex=1.4)


# Simulando uma funcao de verossimilhanca
# Parametros
N=100
s=30
# Probabilidade desconhecida (quero encontrar a de maxima verossimilhança)
p=seq(from=0,to=1,by=0.01)
L=p
for (j in 1:101){
    L[j]=dbinom(s,N,p[j])
}

# Plotando a funcao de verossimilhanca
plot(p,as.numeric(L), ylab="Likelihood",xlab='P(X=1)',
     main="Likelihood function for a binomial process\nwith known parameters N=100, s=30")
abline(v=0.3,col='blue')
#abline(h=max(L),col='red')
new <- data.frame(x = seq(0.2, 0.4, 0.01))
lines(new$x, rep(max(L),times=21),col='red')
